# encoding: utf-8
require 'spec_helper'

describe Shitceptions do
  describe '.configure' do
    it 'yields a configuration block' do
      expect {
        Shitceptions.configure do |c|
        end
      }.to_not raise_error
    end

    context 'block options' do
      shared_examples 'a config_option' do |name, default_value|
        boolean_method = "#{name}?".to_sym
        setter_method  = "#{name}=".to_sym

        it "defaults to #{default_value}" do
          expect(Shitceptions.send(boolean_method)).to eq(default_value)
        end

        it 'accepts user input' do
          expect(Shitceptions).to respond_to(setter_method)
        end

        it 'sets the instance variable to false' do
          Shitceptions.send(setter_method, false)
          expect(Shitceptions.send(boolean_method)).to be_false
        end

        it 'sets the instance variable to true' do
          Shitceptions.send(setter_method, true)
          expect(Shitceptions.send(boolean_method)).to be_true
        end
      end

      describe '#enabled' do
        it_behaves_like 'a config_option', :enabled, true
      end

      describe '#include_original_error' do
        it_behaves_like 'a config_option', :include_original_error, false
      end
    end
  end

  describe '.shittify' do
    it 'raises an exception if a class is not given' do
      expect {
        Shitceptions.shittify("String", nil)
      }.to raise_error(Shitceptions::ShittyArgumentError)
    end

    context 'when #include_original_error is true' do
      let(:klass) do
        class MyClass < StandardError
          def to_s
            "This is the old exception"
          end
        end

        MyClass
      end

      before do
        Shitceptions.include_original_error = true
        Shitceptions.shittify(klass, :pile_of_poo)
      end

      it 'includes the original error' do
        expect {
          raise klass
        }.to raise_error('💩  This is the old exception')
      end
    end

    context 'when #include_original_error is false' do
      let(:klass) do
        class MyClass < StandardError
          def to_s
            "This is the old exception"
          end
        end

        MyClass
      end

      before do
        Shitceptions.include_original_error = false
        Shitceptions.shittify(klass, :pile_of_poo)
      end

      it 'includes the original error' do
        expect {
          raise klass
        }.to raise_error('💩  ')
      end
    end
  end
end
